// src/get-collection-count.ts
import { getCountFromServer, query } from "@react-native-firebase/firestore";
async function getCollectionCount(collectionRef, ...queryConstraints) {
  const snapshot = await getCountFromServer(
    query(collectionRef, ...queryConstraints)
  );
  return snapshot.data().count;
}

// src/get-document.ts
import { doc, getDoc } from "@react-native-firebase/firestore";

// src/utils/invariant.ts
function invariant(condition, msg) {
  if (!condition) {
    throw new Error(msg);
  }
}

// src/utils/is-empty.ts
function isEmpty(array) {
  return array.length === 0;
}

// src/utils/chunk.ts
function chunk(array, size) {
  invariant(size > 0, "Size must be greater than 0");
  if (isEmpty(array)) {
    return [];
  }
  const length = array.length;
  let index = 0;
  let resIndex = 0;
  const result = Array.from({ length: Math.ceil(length / size) });
  while (index < length) {
    result[resIndex++] = array.slice(index, index += size);
  }
  return result;
}

// src/utils/get-error-message.ts
function getErrorMessage(error) {
  return toErrorWithMessage(error).message;
}
function isErrorWithMessage(error) {
  return typeof error === "object" && error !== null && "message" in error && typeof error.message === "string";
}
function toErrorWithMessage(maybeError) {
  if (isErrorWithMessage(maybeError)) return maybeError;
  try {
    return new Error(JSON.stringify(maybeError));
  } catch (e) {
    return new Error(String(maybeError));
  }
}

// src/utils/is-present.ts
function isPresent(t) {
  return t !== void 0 && t !== null;
}
function isDefined(t) {
  return t !== void 0;
}
function isFilled(t) {
  return t !== null;
}

// src/make-mutable-document.ts
import {
  deleteDoc,
  updateDoc
} from "@react-native-firebase/firestore";
function makeMutableDocument(doc3) {
  return {
    id: doc3.id,
    data: doc3.data(),
    ref: doc3.ref,
    update: (data) => updateDoc(doc3.ref, data),
    updateWithPartial: (data) => updateDoc(doc3.ref, data),
    delete: () => deleteDoc(doc3.ref)
  };
}
function makeMutableDocumentInTransaction(doc3, tx) {
  return {
    id: doc3.id,
    data: doc3.data(),
    ref: doc3.ref,
    update: (data) => tx.update(doc3.ref, data),
    updateWithPartial: (data) => tx.update(doc3.ref, data),
    delete: () => tx.delete(doc3.ref)
  };
}

// src/get-document.ts
async function getDocument(collectionRef, documentId) {
  const snapshot = await getDoc(doc(collectionRef, documentId));
  invariant(
    snapshot.exists,
    `No document available at ${collectionRef.path}/${documentId}`
  );
  return makeMutableDocument(snapshot);
}
async function getDocumentMaybe(collectionRef, documentId) {
  const snapshot = await getDoc(doc(collectionRef, documentId));
  if (!snapshot.exists) return;
  return makeMutableDocument(snapshot);
}
async function getDocumentData(collectionRef, documentId) {
  const docSnap = await getDoc(doc(collectionRef, documentId));
  invariant(
    docSnap.exists,
    `No document available at ${collectionRef.path}/${documentId}`
  );
  return docSnap.data();
}
async function getDocumentDataMaybe(collectionRef, documentId) {
  const snapshot = await getDoc(doc(collectionRef, documentId));
  if (!snapshot.exists) return;
  return snapshot.data();
}
async function getDocumentInTransaction(transaction, collectionRef, documentId) {
  const snapshot = await transaction.get(doc(collectionRef, documentId));
  invariant(
    snapshot.exists,
    `No document available at ${collectionRef.path}/${documentId}`
  );
  return makeMutableDocument(snapshot);
}
async function getDocumentInTransactionMaybe(transaction, collectionRef, documentId) {
  const snapshot = await transaction.get(doc(collectionRef, documentId));
  if (!snapshot.exists) {
    return;
  }
  return makeMutableDocument(snapshot);
}

// src/get-specific-document.ts
import { getDoc as getDoc2 } from "@react-native-firebase/firestore";

// src/make-document.ts
function makeDocument(doc3) {
  return {
    id: doc3.id,
    data: doc3.data()
  };
}

// src/get-specific-document.ts
async function getSpecificDocument(documentRef) {
  const snapshot = await getDoc2(documentRef);
  invariant(snapshot.exists, `No document available at ${documentRef.path}`);
  return makeMutableDocument(snapshot);
}
async function getSpecificDocumentData(documentRef) {
  const docSnap = await getDoc2(documentRef);
  invariant(docSnap.exists, `No document available at ${documentRef.path}`);
  return docSnap.data();
}
async function getSpecificDocumentFromTransaction(transaction, documentRef) {
  const snapshot = await transaction.get(documentRef);
  invariant(snapshot.exists, `No document available at ${documentRef.path}`);
  return makeDocument(snapshot);
}

// src/use-collection.ts
import { limit, query as query2 } from "@react-native-firebase/firestore";
import { useMemo as useMemo4 } from "react";

// src/fork/useCollection.ts
import {
  getDocs,
  getDocsFromCache,
  getDocsFromServer,
  onSnapshot
} from "@react-native-firebase/firestore";
import { useCallback as useCallback2, useEffect as useEffect3, useMemo as useMemo2 } from "react";

// src/fork/helpers/isEqualQuery.ts
var isEqualQuery = (v1, v2) => {
  if (!v1 || !v2) {
    return v1 === v2;
  }
  return v1.isEqual(v2);
};

// src/fork/helpers/isEqualRef.ts
var isRefEqual = (v1, v2) => {
  return (v1 == null ? void 0 : v1.path) === (v2 == null ? void 0 : v2.path);
};

// src/fork/helpers/refHooks.ts
import { useEffect, useRef } from "react";
var useComparatorRef = (value, isEqual, onChange) => {
  const ref = useRef(value);
  useEffect(() => {
    if (!isEqual(value, ref.current)) {
      ref.current = value;
      if (onChange) {
        onChange();
      }
    }
  });
  return ref;
};

// src/fork/helpers/useIsFirestoreQueryEqual.ts
var useIsFirestoreQueryEqual = (value, onChange) => {
  return useComparatorRef(value, isEqualQuery, onChange);
};

// src/fork/helpers/useIsFirestoreRefEqual.ts
var useIsFirestoreRefEqual = (value, onChange) => {
  return useComparatorRef(value, isRefEqual, onChange);
};

// src/fork/helpers/useIsMounted.ts
import { useEffect as useEffect2, useState } from "react";
var useIsMounted = () => {
  const [isMounted, setIsMounted] = useState(true);
  useEffect2(() => {
    return () => {
      setIsMounted(false);
    };
  }, []);
  return isMounted;
};

// src/fork/helpers/useLoadingValue.ts
import { useCallback, useMemo, useReducer } from "react";
var defaultState = (defaultValue) => {
  return {
    loading: defaultValue === void 0 || defaultValue === null,
    value: defaultValue
  };
};
var reducer = () => (state, action) => {
  switch (action.type) {
    case "error":
      return {
        ...state,
        error: action.error,
        loading: false,
        value: void 0
      };
    case "reset":
      return defaultState(action.defaultValue);
    case "value":
      return {
        ...state,
        error: void 0,
        loading: false,
        value: action.value
      };
    default:
      return state;
  }
};
function useLoadingValue(getDefaultValue) {
  const defaultValue = getDefaultValue ? getDefaultValue() : void 0;
  const [state, dispatch] = useReducer(
    reducer(),
    defaultState(defaultValue)
  );
  const reset = useCallback(() => {
    const defaultValue2 = getDefaultValue ? getDefaultValue() : void 0;
    dispatch({ type: "reset", defaultValue: defaultValue2 });
  }, [getDefaultValue]);
  const setError = useCallback((error) => {
    dispatch({ type: "error", error });
  }, []);
  const setValue = useCallback((value) => {
    dispatch({ type: "value", value });
  }, []);
  return useMemo(
    () => ({
      error: state.error,
      loading: state.loading,
      reset,
      setError,
      setValue,
      value: state.value
    }),
    [state.error, state.loading, reset, setError, setValue, state.value]
  );
}

// src/fork/useCollection.ts
function useCollection_fork(query4, options) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const ref = useIsFirestoreQueryEqual(query4, reset);
  useEffect3(() => {
    if (!ref.current) {
      setValue(void 0);
      return;
    }
    const unsubscribe = (options == null ? void 0 : options.snapshotListenOptions) ? onSnapshot(
      ref.current,
      options.snapshotListenOptions,
      setValue,
      setError
    ) : onSnapshot(ref.current, setValue, setError);
    return () => {
      unsubscribe();
    };
  }, [ref.current]);
  return [value, loading, error];
}
function useCollectionOnce_fork(query4, options) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const isMounted = useIsMounted();
  const ref = useIsFirestoreQueryEqual(query4, reset);
  const loadData = useCallback2(
    async (query5, options2) => {
      if (!query5) {
        setValue(void 0);
        return;
      }
      const get = getDocsFnFromGetOptions(options2 == null ? void 0 : options2.getOptions);
      try {
        const result = await get(query5);
        if (isMounted) {
          setValue(result);
        }
      } catch (error2) {
        if (isMounted) {
          setError(error2);
        }
      }
    },
    []
  );
  const reloadData = useCallback2(
    () => loadData(ref.current, options),
    [loadData, ref.current]
  );
  useEffect3(() => {
    loadData(ref.current, options).catch(setError);
  }, [ref.current]);
  return [value, loading, error, reloadData];
}
var getDocsFnFromGetOptions = ({ source } = { source: "default" }) => {
  switch (source) {
    default:
    case "default":
      return getDocs;
    case "cache":
      return getDocsFromCache;
    case "server":
      return getDocsFromServer;
  }
};

// src/fork/useDocument.ts
import {
  getDoc as getDoc3,
  getDocFromCache,
  getDocFromServer,
  onSnapshot as onSnapshot2
} from "@react-native-firebase/firestore";
import { useCallback as useCallback3, useEffect as useEffect4, useMemo as useMemo3 } from "react";
var useDocument_fork = (docRef, options) => {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const ref = useIsFirestoreRefEqual(docRef, reset);
  useEffect4(() => {
    if (!ref.current) {
      setValue(void 0);
      return;
    }
    const unsubscribe = (options == null ? void 0 : options.snapshotListenOptions) ? onSnapshot2(
      ref.current,
      options.snapshotListenOptions,
      setValue,
      (err) => {
        if (err instanceof Error) {
          setError(err);
        }
      }
    ) : onSnapshot2(ref.current, setValue, (err) => {
      if (err instanceof Error) {
        setError(err);
      }
    });
    return () => {
      unsubscribe();
    };
  }, [ref.current]);
  return [value, loading, error];
};
function useDocumentOnce_fork(docRef, options) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const isMounted = useIsMounted();
  const ref = useIsFirestoreRefEqual(docRef, reset);
  const loadData = useCallback3(
    async (reference, options2) => {
      if (!reference) {
        setValue(void 0);
        return;
      }
      const get = getDocFnFromGetOptions(options2 == null ? void 0 : options2.getOptions);
      try {
        const result = await get(reference);
        if (isMounted) {
          setValue(result);
        }
      } catch (err) {
        if (isMounted) {
          if (err instanceof Error) {
            setError(err);
          }
        }
      }
    },
    []
  );
  const reloadData = useCallback3(
    () => loadData(ref.current, options),
    [loadData, ref.current]
  );
  useEffect4(() => {
    if (!ref.current) {
      setValue(void 0);
      return;
    }
    loadData(ref.current, options).catch((err) => {
      if (err instanceof Error) {
        setError(err);
      }
    });
  }, [ref.current]);
  return [value, loading, error, reloadData];
}
var getDocFnFromGetOptions = ({ source } = { source: "default" }) => {
  switch (source) {
    default:
    case "default":
      return getDoc3;
    case "cache":
      return getDocFromCache;
    case "server":
      return getDocFromServer;
  }
};

// src/use-collection.ts
function useCollection(collectionRef, ...queryConstraints) {
  const hasNoConstraints = queryConstraints.length === 0;
  const _query = hasNoConstraints ? query2(collectionRef, limit(500)) : query2(collectionRef, ...queryConstraints.filter(isDefined));
  const [snapshot, , error] = useCollection_fork(_query);
  if (error) {
    throw new Error(
      `Failed to execute query on ${collectionRef.path}: ${getErrorMessage(error)}`
    );
  }
  const docs = useMemo4(() => {
    if (!snapshot) {
      return void 0;
    }
    return snapshot.docs.map((doc3) => makeMutableDocument(doc3));
  }, [snapshot]);
  return docs ? [docs, false] : [void 0, true];
}
function useCollectionOnce(collectionRef, ...queryConstraints) {
  const hasNoConstraints = queryConstraints.length === 0;
  const _query = hasNoConstraints ? query2(collectionRef, limit(500)) : query2(collectionRef, ...queryConstraints.filter(isDefined));
  const [snapshot, , error] = useCollectionOnce_fork(_query);
  if (error) {
    throw new Error(
      `Failed to execute query on ${collectionRef.path}: ${getErrorMessage(error)}`
    );
  }
  const docs = useMemo4(() => {
    if (!snapshot) {
      return void 0;
    }
    return snapshot.docs.map((doc3) => makeMutableDocument(doc3));
  }, [snapshot]);
  return docs ? [docs, false] : [void 0, true];
}
function useCollectionMaybe(collectionRef, ...queryConstraints) {
  const hasNoConstraints = queryConstraints.length === 0;
  const _query = hasNoConstraints ? query2(collectionRef, limit(500)) : query2(collectionRef, ...queryConstraints.filter(isDefined));
  const [snapshot, isLoading] = useCollection_fork(_query);
  const docs = useMemo4(() => {
    if (!snapshot) {
      return void 0;
    }
    return snapshot.docs.map((doc3) => makeMutableDocument(doc3));
  }, [snapshot]);
  return [docs, isLoading];
}

// src/use-collection-count.ts
import { useCallback as useCallback4 } from "react";
import {
  getCountFromServer as getCountFromServer2,
  query as query3
} from "@react-native-firebase/firestore";
import { useEffect as useEffect5 } from "react";
function useCollectionCount(query4) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const isMounted = useIsMounted();
  const ref = useIsFirestoreQueryEqual(query4, reset);
  const loadData = useCallback4(async (query5) => {
    if (!query5) {
      setValue(void 0);
      return;
    }
    try {
      const result = await getCountFromServer2(query5);
      if (isMounted) {
        setValue(result.data().count);
      }
    } catch (error2) {
      if (isMounted) {
        setError(error2);
      }
    }
  }, []);
  const reloadData = useCallback4(
    () => loadData(ref.current),
    [loadData, ref.current]
  );
  useEffect5(() => {
    loadData(ref.current).catch(setError);
  }, [ref.current]);
  return [value, loading, error, reloadData];
}
function useCollectionCountOnce(collectionRef, ...queryConstraints) {
  const [value, , error] = useCollectionCount(
    query3(collectionRef, ...queryConstraints.filter(isDefined))
  );
  if (error) {
    throw new Error(
      `Failed to execute query on ${collectionRef.path}: ${getErrorMessage(error)}`
    );
  }
  return value !== void 0 ? [value, false] : [void 0, true];
}

// src/use-document.ts
import { doc as doc2 } from "@react-native-firebase/firestore";
import { useMemo as useMemo5 } from "react";
function useDocument(collectionRef, documentId) {
  const ref = documentId ? doc2(collectionRef, documentId) : void 0;
  const [snapshot, , error] = useDocument_fork(ref);
  if (error) {
    throw error;
  }
  const document = useMemo5(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useDocumentMaybe(collectionRef, documentId) {
  const ref = documentId ? doc2(collectionRef, documentId) : void 0;
  const [snapshot, isLoading] = useDocument_fork(ref);
  const document = useMemo5(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return [document, isLoading];
}
function useDocumentData(collectionRef, documentId) {
  const [document, isLoading] = useDocument(collectionRef, documentId);
  return isLoading ? [void 0, true] : [document.data, false];
}
function useDocumentDataMaybe(collectionRef, documentId) {
  const [document, isLoading] = useDocumentMaybe(collectionRef, documentId);
  return [document == null ? void 0 : document.data, isLoading];
}
function useDocumentOnce(collectionRef, documentId) {
  const ref = documentId ? doc2(collectionRef, documentId) : void 0;
  const [snapshot, , error] = useDocumentOnce_fork(ref);
  if (error) {
    throw error;
  }
  const document = useMemo5(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useDocumentDataOnce(collectionRef, documentId) {
  const [document, isLoading] = useDocumentOnce(collectionRef, documentId);
  return isLoading ? [void 0, true] : [document.data, false];
}

// src/use-specific-document.ts
import { useMemo as useMemo6 } from "react";
function useSpecificDocument(documentRef) {
  const [snapshot, , error] = useDocument_fork(documentRef);
  if (error) {
    throw error;
  }
  const document = useMemo6(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useSpecificDocumentMaybe(documentRef) {
  const [snapshot, isLoading] = useDocument_fork(documentRef);
  const document = useMemo6(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return [document, isLoading];
}
function useSpecificDocumentData(documentRef) {
  const [document, isLoading] = useSpecificDocument(documentRef);
  return isLoading ? [void 0, true] : [document.data, false];
}
function useSpecificDocumentDataMaybe(documentRef) {
  const [document, isLoading] = useSpecificDocumentMaybe(documentRef);
  return [document == null ? void 0 : document.data, isLoading];
}
function useSpecificDocumentOnce(documentRef) {
  const [snapshot, , error] = useDocumentOnce_fork(documentRef);
  if (error) {
    throw error;
  }
  const document = useMemo6(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useSpecificDocumentDataOnce(documentRef) {
  const [document, isLoading] = useSpecificDocumentOnce(documentRef);
  return isLoading ? [void 0, true] : [document.data, false];
}
export {
  chunk,
  getCollectionCount,
  getDocument,
  getDocumentData,
  getDocumentDataMaybe,
  getDocumentInTransaction,
  getDocumentInTransactionMaybe,
  getDocumentMaybe,
  getErrorMessage,
  getSpecificDocument,
  getSpecificDocumentData,
  getSpecificDocumentFromTransaction,
  invariant,
  isDefined,
  isEmpty,
  isFilled,
  isPresent,
  makeDocument,
  makeMutableDocument,
  makeMutableDocumentInTransaction,
  useCollection,
  useCollectionCountOnce,
  useCollectionMaybe,
  useCollectionOnce,
  useDocument,
  useDocumentData,
  useDocumentDataMaybe,
  useDocumentDataOnce,
  useDocumentMaybe,
  useDocumentOnce,
  useSpecificDocument,
  useSpecificDocumentData,
  useSpecificDocumentDataMaybe,
  useSpecificDocumentDataOnce,
  useSpecificDocumentMaybe,
  useSpecificDocumentOnce
};
//# sourceMappingURL=index.mjs.map